/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/

package org.fortiss.af3.rcp.application.advisors;

import static org.eclipse.wb.swt.ResourceManager.getPluginImageDescriptor;
import static org.fortiss.tooling.kernel.ui.util.TutorialUIServiceUtils.createTutorialMenu;

import java.util.Map;

import org.eclipse.core.commands.IHandler;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.internal.actions.CommandAction;
import org.eclipse.ui.internal.ide.IDEWorkbenchMessages;

/**
 * Action bar advisor for AF3 RCP.
 * 
 * @author ratiu
 */

public class ApplicationActionBarAdvisor extends ActionBarAdvisor {

	// standard Eclipse actions

	/** Standard Eclipse Preferences Action. */
	private IWorkbenchAction openPreferencesAction;

	/** Standard Eclipse New-Wizard Action. */
	private IAction newWizardDropDownAction;

	/** Standard Eclipse Save Action. */
	private IAction saveAction;

	/** Standard Eclipse Quit Action. */
	private IAction quitAction;

	/** Standard Eclipse Undo Action. */
	private IAction undoAction;

	/** Standard Eclipse Redo Action. */
	private IAction redoAction;

	/** Standard Eclipse Back History Action. */
	private IAction backAction;

	/** Standard Eclipse Back History Action. */
	private IAction forwardAction;

	/** Standard Eclipse Cut Action. */
	private IAction cutAction;

	/** Standard Eclipse Copy Action. */
	private IAction copyAction;

	/** Standard Eclipse PAste Action. */
	private IAction pasteAction;

	/** Standard Eclipse Delete Action. */
	private IAction deleteAction;

	/** Standard Eclipse Help. */
	private IAction helpContentsAction;

	/** Standard Eclipse Welcome Action. */
	private IAction introAction;

	/** Standard Eclipse About Action. */
	private IAction aboutAction;

	/** Refresh the workspace. */
	private static WorkbenchCommandAction refreshWorkspaceAction;

	/** Refresh the workbench. */
	private static WorkbenchCommandAction exitWorkbenchAction;

	// AF3 specific actions and labels

	/** AF3-Specific Action to crate a new empty project. */
	private static WorkbenchCommandAction newAF3ProjectAction;

	/** AF3-Specific Action to crate a new empty library. */
	private static WorkbenchCommandAction newAF3LibraryAction;

	/** Open the "goBack" view. */
	private static IWorkbenchAction goBack;

	/** Open the "goForward" view. */
	private static IWorkbenchAction goForward;

	/** Open the "Error Log". */
	private static WorkbenchCommandAction showErrorLogAction;

	/** Open the "Model Storage Error Dialog". */
	private static WorkbenchCommandAction showModelStorageErrorDialogAction;

	/** AF3-Specific Import Action. */
	private static WorkbenchCommandAction importAF3ProjectsAction;

	/** AF3-Specific Export Action. */
	private static WorkbenchCommandAction exportAF3ProjectsAction;

	/** The current objective combo box. */
	private static CurrentObjectiveContributionItem currentObjective;

	/** The upcoming objective combo box. */
	private static UpcomingObjectiveContributionItem upcomingObjective;

	// Actions to open Examples.

	/** AF3-Specific Action to open the SimpleTrafficLights-Example. */
	private static WorkbenchCommandAction openSimpleTrafficLightsExampleAction;

	/** AF3-Specific Action to open the STL-Example for mode demonstration. */
	private static WorkbenchCommandAction openSimpleTrafficLightsModesExampleAction;

	/** AF3-Specific Action to open the STL-Example for code generation. */
	private static WorkbenchCommandAction openSimpleTrafficLightsCodeGenerationExampleAction;

	/** AF3-Specific Action to open the SimpleTrafficLights_AF3Tour-Example. */
	private static WorkbenchCommandAction openSimpleTrafficLightsAF3TourExampleAction;

	/** AF3-Specific Action to open the MotorManagement-Example. */
	private static WorkbenchCommandAction openMotorManagementExampleAction;

	/** AF3-Specific Action to open the ACC-Example. */
	private static WorkbenchCommandAction openACCExampleAction;

	/** AF3-Specific Action to open the eCar-Example. */
	private static WorkbenchCommandAction openECarExampleAction;

	/** AF3-Specific Action to open the EmergencyStop-Example. */
	private static WorkbenchCommandAction openEmergencyStopExampleAction;

	/** AF3-Specific Action to open the EmergencyStopWeaklyCausal-Example. */
	private static WorkbenchCommandAction openEmergencyStopWeaklyCausalExampleAction;

	/** AF3-Specific Action to open the Beagle Java Deployment Example. */
	private static WorkbenchCommandAction openSTLCBeagleJavaDeploymentExampleAction;

	/** AF3-Specific Action to open the operator panel example. */
	private static WorkbenchCommandAction openOperatorPanelExampleAction;

	/**
	 * Class used to define an command action.
	 * 
	 * TODO (Issue #3140): {@link CommandAction} is now deprecated.
	 * Menu contributions should be used instead. This probably requires a refactoring of the
	 * complete class. It is however questionable whether this refactoring is worth it since we
	 * should soon switch to SystemFocus.
	 */
	private static class WorkbenchCommandAction extends CommandAction implements IWorkbenchAction {

		/** Constructor. */
		public WorkbenchCommandAction(String commandIdIn, Map<?, ?> parameterMap,
				IWorkbenchWindow window) {
			super(window, commandIdIn, parameterMap);
		}
	}

	/** Constructor. */
	public ApplicationActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
	}

	/** {@inheritDoc} */
	@Override
	protected void makeActions(IWorkbenchWindow window) {

		newWizardDropDownAction = ActionFactory.NEW_WIZARD_DROP_DOWN.create(window);
		register(newWizardDropDownAction);
		saveAction = ActionFactory.SAVE.create(window);
		register(saveAction);
		openPreferencesAction = ActionFactory.PREFERENCES.create(window);
		register(openPreferencesAction);
		// Actions for invisible accelerators.
		quitAction = ActionFactory.QUIT.create(window);
		register(quitAction);
		undoAction = ActionFactory.UNDO.create(window);
		register(undoAction);
		redoAction = ActionFactory.REDO.create(window);
		register(redoAction);
		backAction = ActionFactory.BACKWARD_HISTORY.create(window);
		register(backAction);
		forwardAction = ActionFactory.FORWARD_HISTORY.create(window);
		register(forwardAction);
		cutAction = ActionFactory.CUT.create(window);
		register(cutAction);
		copyAction = ActionFactory.COPY.create(window);
		register(copyAction);
		pasteAction = ActionFactory.PASTE.create(window);
		register(pasteAction);
		deleteAction = ActionFactory.DELETE.create(window);
		register(deleteAction);

		registerCreateNewProject(window);
		registerCreateNewLibrary(window);
		registerLoadSimpleTrafficLightsExample(window);
		registerLoadSimpleTrafficLightsModesExample(window);
		registerLoadSimpleTrafficLightsCodeGenerationExample(window);
		registerLoadSimpleTrafficLightsAF3TourExample(window);
		registerLoadMotorManagementExample(window);
		registerLoadACCExample(window);
		registerLoadeCarExample(window);
		registerEmergencyStopExample(window);
		registerEmergencyStopWeaklyCausalExample(window);
		registerLoadOperatorPanelExample(window);
		registerLoadSTLCBeagleJavaDeploymentExample(window);
		registerImportAF3Project(window);
		registerExportAF3Project(window);
		registerExit(window);
		registerErrorLog(window);
		registerStorageErrorDialog(window);
		registerRefreshWorkspace(window);
		registerHelpContentsAction(window);
		registerIntroAction(window);
		registerAboutAction(window);
		registerGoBack(window);
		registerGoForward(window);
		createCurrentObjective();
		createUpcomingObjective();
	}

	/**
	 * Creates and registers a {@link WorkbenchCommandAction} with a given {@code actionText}.
	 * 
	 * @param actionText
	 *            Text describing the new {@link WorkbenchCommandAction}. It is also as tooltip text
	 *            and description, and to derive the {@link WorkbenchCommandAction}'s unique ID (not
	 *            be confused with {@code commandId}.
	 * @param commandId
	 *            ID linking the {@link WorkbenchCommandAction} to be created with an
	 *            {@link IHandler} that performs the actual work.
	 * @param window
	 *            Workbench window for which the {@link WorkbenchCommandAction} is be created.
	 * @param imageDescriptor
	 *            Image for new {@link WorkbenchCommandAction} (may be {@code null}).
	 * @param parameterMap
	 *            {@link Map} of parameters passed to {@link WorkbenchCommandAction}.
	 * @return {@link WorkbenchCommandAction} with a given {@code actionText}.
	 */
	private WorkbenchCommandAction createAndRegisterWorkBenchCommandAction(String actionText,
			String commandId, ImageDescriptor imageDescriptor, Map<?, ?> parameterMap,
			IWorkbenchWindow window) {

		WorkbenchCommandAction workbenchCommandAction =
				new WorkbenchCommandAction(commandId, parameterMap, window);

		workbenchCommandAction.setText(actionText);

		String actionTextWithoutShortCuts = actionText.replaceAll(",", "");
		workbenchCommandAction.setToolTipText(actionTextWithoutShortCuts);
		workbenchCommandAction.setDescription(actionTextWithoutShortCuts);
		workbenchCommandAction.setId(actionTextWithoutShortCuts.replaceAll(" ", ""));
		if(imageDescriptor != null) {
			workbenchCommandAction.setImageDescriptor(imageDescriptor);
		}
		register(workbenchCommandAction);

		return workbenchCommandAction;
	}

	/**
	 * Creates and registers a {@link WorkbenchCommandAction} with a given {@code actionText}.
	 * 
	 * @param actionText
	 *            Text describing the new {@link WorkbenchCommandAction}. It is also as tooltip text
	 *            and description, and to derive the {@link WorkbenchCommandAction}'s unique ID (not
	 *            be confused with {@code commandId}.
	 * @param commandId
	 *            ID linking the {@link WorkbenchCommandAction} to be created with an
	 *            {@link IHandler} that performs the actual work.
	 * @param window
	 *            Workbench window for which the {@link WorkbenchCommandAction} is be created.
	 * @return {@link WorkbenchCommandAction} with a given {@code actionText}.
	 */
	private WorkbenchCommandAction createAndRegisterWorkBenchCommandAction(String actionText,
			String commandId, IWorkbenchWindow window) {

		return createAndRegisterWorkBenchCommandAction(actionText, commandId, null, null, window);
	}

	/**
	 * Registers the "About AutoFOCUS3" action and attaches it to the corresponding button.
	 */
	private void registerAboutAction(IWorkbenchWindow window) {
		aboutAction = ActionFactory.ABOUT.create(window);
		aboutAction.setId("aboutAction");
		aboutAction.setImageDescriptor(getPluginImageDescriptor("org.fortiss.af3.rcp.application",
				"icons/dock_icons/icon16.png"));
		aboutAction.setText("&About AutoFOCUS3");
		aboutAction.setToolTipText("About AutoFOCUS3");
		aboutAction.setDescription("About AutoFOCUS3");
		register(aboutAction);
	}

	/**
	 * Registers the "Intro" action and attaches it to the corresponding button.
	 */
	private void registerIntroAction(IWorkbenchWindow window) {
		introAction = ActionFactory.INTRO.create(window);
		register(introAction);
	}

	/**
	 * Registers the "AF3 Help" action and attaches it to the corresponding button.
	 */
	private void registerHelpContentsAction(IWorkbenchWindow window) {
		helpContentsAction = ActionFactory.HELP_CONTENTS.create(window);
		helpContentsAction.setId("helpContentsAction");
		helpContentsAction.setToolTipText("AF3 Help");
		helpContentsAction.setText("AF3 &Help");
		register(helpContentsAction);
	}

	/**
	 * Registers the "New AF3 Project" action and attaches it to the corresponding button.
	 */
	private void registerCreateNewProject(IWorkbenchWindow window) {
		newAF3ProjectAction = createAndRegisterWorkBenchCommandAction("New AF3 Project",
				"org.fortiss.af3.rcp.application.newaf3project", getPluginImageDescriptor(
						"org.fortiss.af3.rcp.application", "icons/new_project.png"),
				null, window);
	}

	/**
	 * Registers the "New AF3 Library" action and attaches it to the corresponding button.
	 */
	private void registerCreateNewLibrary(IWorkbenchWindow window) {
		newAF3LibraryAction = createAndRegisterWorkBenchCommandAction("New AF3 Library",
				"org.fortiss.af3.rcp.application.newaf3library", getPluginImageDescriptor(
						"org.fortiss.af3.rcp.application", "icons/new_library.png"),
				null, window);
	}

	/**
	 * Registers the "Load Simple Traffic Lights Example" action and attaches it to the
	 * corresponding button.
	 */
	private void registerLoadSimpleTrafficLightsExample(IWorkbenchWindow window) {
		openSimpleTrafficLightsExampleAction =
				createAndRegisterWorkBenchCommandAction("Load Simple Traffic Lights Example",
						"org.fortiss.af3.rcp.application.loadsimpletrafficlightsexample", window);
	}

	/**
	 * Registers the "Load Simple Traffic Lights Modes Example" action and attaches it to the
	 * corresponding button.
	 */
	private void registerLoadSimpleTrafficLightsModesExample(IWorkbenchWindow window) {
		openSimpleTrafficLightsModesExampleAction = createAndRegisterWorkBenchCommandAction(
				"Load Simple Traffic Lights Modes Example",
				"org.fortiss.af3.rcp.application.loadsimpletrafficlightsmodesexample", window);
	}

	/**
	 * Registers the "Load Simple Traffic Lights Code Generation Example" action and attaches it to
	 * the corresponding button.
	 */
	private void registerLoadSimpleTrafficLightsCodeGenerationExample(IWorkbenchWindow window) {
		openSimpleTrafficLightsCodeGenerationExampleAction =
				createAndRegisterWorkBenchCommandAction(
						"Load Simple Traffic Lights Code Generation Example",
						"org.fortiss.af3.rcp.application.loadsimpletrafficlightscodegenerationexample",
						window);
	}

	/**
	 * Registers the "Load Simple Traffic Lights AF3 Tour Example" action and attaches it to the
	 * corresponding button.
	 */
	private void registerLoadSimpleTrafficLightsAF3TourExample(IWorkbenchWindow window) {
		openSimpleTrafficLightsAF3TourExampleAction = createAndRegisterWorkBenchCommandAction(
				"Load Simple Traffic Lights AF3 Tour Example",
				"org.fortiss.af3.rcp.application.loadsimpletrafficlightsaf3tourexample", window);
	}

	/**
	 * Registers the "Load Motor-Management Example" action and attaches it to the corresponding
	 * button.
	 */
	private void registerLoadMotorManagementExample(IWorkbenchWindow window) {
		openMotorManagementExampleAction =
				createAndRegisterWorkBenchCommandAction("Load Motor-Management Example",
						"org.fortiss.af3.rcp.application.loadmotormanagementexample", window);
	}

	/**
	 * Registers the "Load ACC Example" action and attaches it to the corresponding button.
	 */
	private void registerLoadACCExample(IWorkbenchWindow window) {
		openACCExampleAction = createAndRegisterWorkBenchCommandAction("Load ACC Example",
				"org.fortiss.af3.rcp.application.loadaccexample", window);
	}

	/**
	 * Registers the "Load eCar Example" action and attaches it to the corresponding button.
	 */
	private void registerLoadeCarExample(IWorkbenchWindow window) {
		openECarExampleAction = createAndRegisterWorkBenchCommandAction("Load eCar Example",
				"org.fortiss.af3.rcp.application.loadecarexample", window);
	}

	/**
	 * Registers the "Load Emergency-Stop Example" action and attaches it to the corresponding
	 * button.
	 */
	private void registerEmergencyStopExample(IWorkbenchWindow window) {
		openEmergencyStopExampleAction =
				createAndRegisterWorkBenchCommandAction("Load Emergency-Stop Example",
						"org.fortiss.af3.rcp.application.loademergencystopexample", window);
	}

	/**
	 * Registers the "Load Emergency-Stop Weakly Causal Example" action and attaches it to the
	 * corresponding button.
	 */
	private void registerEmergencyStopWeaklyCausalExample(IWorkbenchWindow window) {
		openEmergencyStopWeaklyCausalExampleAction = createAndRegisterWorkBenchCommandAction(
				"Load Emergency-Stop Weakly Causal Example",
				"org.fortiss.af3.rcp.application.loademergencystopweaklycausalexample", window);
	}

	/**
	 * Registers the "Load Operator Panel Example" action and attaches it to the corresponding
	 * button.
	 */
	private void registerLoadOperatorPanelExample(IWorkbenchWindow window) {
		openOperatorPanelExampleAction =
				createAndRegisterWorkBenchCommandAction("Load Operator Panel Example",
						"org.fortiss.af3.rcp.application.loadOperatorPanelExample", window);
	}

	/**
	 * Registers the "Load STLC Beagle Java Deployment Example" action and attaches it to the
	 * corresponding button.
	 */
	private void registerLoadSTLCBeagleJavaDeploymentExample(IWorkbenchWindow window) {
		openSTLCBeagleJavaDeploymentExampleAction = createAndRegisterWorkBenchCommandAction(
				"Load STLC Beagle Java Deployment Example",
				"org.fortiss.af3.rcp.application.loadSTLCBeagleJavaDeploymentExample", window);
	}

	/**
	 * Registers the "Import AF3-Project" action and attaches it to the corresponding button.
	 */
	private void registerImportAF3Project(IWorkbenchWindow window) {
		importAF3ProjectsAction = createAndRegisterWorkBenchCommandAction("&Import AF3-Project",
				"org.fortiss.af3.rcp.application.importaf3project",
				getPluginImageDescriptor("org.eclipse.ui", "/icons/full/etool16/import_wiz.gif"),
				null, window);
	}

	/**
	 * Registers the "Export AF3-Project" action and attaches it to the corresponding button.
	 */
	private void registerExportAF3Project(IWorkbenchWindow window) {
		exportAF3ProjectsAction = createAndRegisterWorkBenchCommandAction("&Export AF3-Project",
				"org.fortiss.af3.rcp.application.exportaf3project",
				getPluginImageDescriptor("org.eclipse.ui", "/icons/full/etool16/export_wiz.gif"),
				null, window);
	}

	/**
	 * Registers the "Exit" action and attaches it to the corresponding button.
	 */
	private void registerExit(IWorkbenchWindow window) {
		exitWorkbenchAction = createAndRegisterWorkBenchCommandAction("Exit",
				"org.fortiss.af3.rcp.application.af3exit", window);
	}

	/**
	 * Registers the "Show Error Log" action and attaches it to the corresponding button.
	 */
	private void registerErrorLog(IWorkbenchWindow window) {
		showErrorLogAction = createAndRegisterWorkBenchCommandAction("Show Error Log",
				"org.fortiss.af3.rcp.application.showErrorLogAction",
				getPluginImageDescriptor("org.fortiss.af3.rcp.application", "icons/error_log.gif"),
				null, window);
	}

	/**
	 * Registers the "Show Model Storage Errors" action and attaches it to the corresponding button.
	 */
	private void registerStorageErrorDialog(IWorkbenchWindow window) {
		showModelStorageErrorDialogAction =
				createAndRegisterWorkBenchCommandAction("Show Model Storage Errors",
						"org.fortiss.af3.rcp.application.showModelStorageErrorDialogAction",
						getPluginImageDescriptor("org.fortiss.af3.rcp.application",
								"icons/storage_error_log.gif"),
						null, window);
	}

	/**
	 * Registers the "Refresh Workspace" action and attaches it to the corresponding button.
	 */
	private void registerRefreshWorkspace(IWorkbenchWindow window) {
		refreshWorkspaceAction = createAndRegisterWorkBenchCommandAction("Refresh Workspace",
				"org.fortiss.af3.rcp.application.refreshWorkspaceAction",
				getPluginImageDescriptor("org.fortiss.af3.rcp.application", "icons/refresh.gif"),
				null, window);
	}

	/**
	 * Registers the "Go Forward" action and attaches it to the corresponding
	 * button.
	 */
	private void registerGoForward(IWorkbenchWindow window) {
		goForward = ActionFactory.FORWARD_HISTORY.create(window);
		register(forwardAction);
		register(goForward);
	}

	/**
	 * Registers the "Go Back" action and attaches it to the corresponding
	 * button.
	 */
	private void registerGoBack(IWorkbenchWindow window) {
		goBack = ActionFactory.BACKWARD_HISTORY.create(window);
		register(backAction);
		register(goBack);
	}

	/**
	 * Creates the "current objective" label by loading the text from a file
	 */
	private void createCurrentObjective() {
		currentObjective = new CurrentObjectiveContributionItem();
	}

	/**
	 * Creates the "upcoming objective" label by loading the text from a file
	 * (for better alignment with the other toolbar elements)
	 */
	private void createUpcomingObjective() {
		upcomingObjective = new UpcomingObjectiveContributionItem();
	}

	/** {@inheritDoc} */
	@Override
	protected void fillCoolBar(ICoolBarManager coolBar) {
		ToolBarManager toolBarManager = new ToolBarManager();
		coolBar.add(toolBarManager);
		toolBarManager.add(newAF3ProjectAction);
		toolBarManager.add(newAF3LibraryAction);
		toolBarManager.add(new Separator());
		toolBarManager.add(saveAction);
		toolBarManager.add(new Separator());
		toolBarManager.add(goBack);
		toolBarManager.add(goForward);
		toolBarManager.add(new Separator());
		toolBarManager.add(currentObjective);
		toolBarManager.add(upcomingObjective);

		upcomingObjective.setCurrentObjective(currentObjective);
	}

	/** {@inheritDoc} */
	@Override
	protected void fillMenuBar(IMenuManager menuBar) {
		// FileMenu.
		MenuManager fileMenu = new MenuManager(IDEWorkbenchMessages.Workbench_file,
				IWorkbenchActionConstants.M_FILE);
		menuBar.add(fileMenu);
		fileMenu.add(newAF3ProjectAction);
		fileMenu.add(newAF3LibraryAction);

		MenuManager openAF3ExampleMenuManager = new MenuManager("Open AF3 Example");
		fileMenu.add(openAF3ExampleMenuManager);
		openAF3ExampleMenuManager.add(openSimpleTrafficLightsExampleAction);
		openAF3ExampleMenuManager.add(openSimpleTrafficLightsModesExampleAction);
		openAF3ExampleMenuManager.add(openSimpleTrafficLightsCodeGenerationExampleAction);
		openAF3ExampleMenuManager.add(openSimpleTrafficLightsAF3TourExampleAction);
		openAF3ExampleMenuManager.add(new Separator());
		openAF3ExampleMenuManager.add(openMotorManagementExampleAction);
		openAF3ExampleMenuManager.add(openACCExampleAction);
		openAF3ExampleMenuManager.add(openECarExampleAction);
		openAF3ExampleMenuManager.add(openEmergencyStopExampleAction);
		openAF3ExampleMenuManager.add(openEmergencyStopWeaklyCausalExampleAction);
		openAF3ExampleMenuManager.add(openOperatorPanelExampleAction);
		openAF3ExampleMenuManager.add(openSTLCBeagleJavaDeploymentExampleAction);
		fileMenu.add(new Separator());
		fileMenu.add(refreshWorkspaceAction);
		fileMenu.add(saveAction);
		fileMenu.add(new Separator());
		fileMenu.add(importAF3ProjectsAction);
		fileMenu.add(exportAF3ProjectsAction);
		fileMenu.add(new Separator());
		fileMenu.add(exitWorkbenchAction);

		// EditMenu.
		MenuManager editMenu = new MenuManager(IDEWorkbenchMessages.Workbench_edit,
				IWorkbenchActionConstants.M_EDIT);
		menuBar.add(editMenu);
		editMenu.add(undoAction);
		editMenu.add(redoAction);
		editMenu.add(backAction);
		editMenu.add(forwardAction);
		editMenu.add(new Separator());
		editMenu.add(cutAction);
		editMenu.add(copyAction);
		editMenu.add(pasteAction);
		editMenu.add(new Separator());
		editMenu.add(deleteAction);
		// Required to make text editors open on first attempt in storage view
		editMenu.add(new Separator(IWorkbenchActionConstants.FIND_EXT));

		// TutorialMenu
		MenuManager tutorialMenu = createTutorialMenu();
		menuBar.add(tutorialMenu);

		// Help Menu
		MenuManager helpMenu = new MenuManager("&Help", IWorkbenchActionConstants.M_HELP);
		menuBar.add(helpMenu);
		helpMenu.add(introAction);
		helpMenu.add(aboutAction);
		helpMenu.add(helpContentsAction);
		helpMenu.add(showErrorLogAction);
		helpMenu.add(showModelStorageErrorDialogAction);
	}

	/** Returns defined action to create new AF3 Library. */
	public static WorkbenchCommandAction getNewAF3LibraryAction() {
		return newAF3LibraryAction;
	}

	/** Returns defined action to create new AF3 Project. */
	public static WorkbenchCommandAction getNewAF3ProjectAction() {
		return newAF3ProjectAction;
	}

	/** Returns openSimpleTrafficLightsExampleAction. */
	public static WorkbenchCommandAction getOpenSimpleTrafficLightsExampleAction() {
		return openSimpleTrafficLightsExampleAction;
	}

	/** Returns openSimpleTrafficLightsModesExampleAction. */
	public static WorkbenchCommandAction getOpenSimpleTrafficLightsModesExampleAction() {
		return openSimpleTrafficLightsModesExampleAction;
	}

	/** Returns openSimpleTrafficLightsCodeGenerationExampleAction. */
	public static WorkbenchCommandAction getOpenSimpleTrafficLightsCodeGenerationExampleAction() {
		return openSimpleTrafficLightsCodeGenerationExampleAction;
	}
}
